import greenfoot.*;

/**
 * The world where you can really play the game. It is a presented world,
 * so you can't just use it without another world.
 * 
 * @author Sven van Nigtevecht
 * @version 1.0
 */
public class GameWorld extends PresentedWorld implements ButtonResponder, PauseMenuResponder
{
    private static final String BACK_BUTTON_TITLE = "Main Menu";
    private static final String REPLAY_BUTTON_TITLE = "Replay";
    private static final String PAUSE_BUTTON_TAG = "Pause";
    
    private int numberOfBalls;
    
    private Counter counter;
    private Paddle paddle;
    private Player player;
    
    private PauseButton pauseButton;
    private PauseMenu pauseMenu;
    private Saver saver = new Saver();
    
    private Thread highScoreThread;
    
    /**
     * Constructor for objects of class GameWorld.
     */
    public GameWorld(int width, int height, int cellSize, GreenfootImage background)
    {
        super(width, height, cellSize, background);
        setPaintOrder(Button.class, Text.class, PauseMenu.class, Message.class,
            Counter.class, ScoreBoard.class,
            Player.class, Paddle.class);
        createUI();
    }
    
    private void createUI()
    {
        createBalls();
        
        this.paddle = new Paddle();
        addObject(this.paddle, getWidth() /2, getHeight() -100);
        this.player = this.paddle.getBall();
        
        this.counter = new Counter();
        GreenfootImage img = this.counter.getImage();
        addObject(this.counter, img.getWidth() /2 +10, getHeight() -img.getHeight() /2 -15);
        
        this.pauseButton = new PauseButton(PAUSE_BUTTON_TAG);
        this.pauseButton.setResponder(this);
        addObject(this.pauseButton, this.counter.getX(), this.counter.getY() -44);
    }
    
    private void createBalls()
    {
        Ball b = new Ball();
        int numBalls = getWidth() /b.getImage().getWidth();
        for (int h = 0; h < 3; h++) {
            int y = b.getImage().getHeight() /2 + h* b.getImage().getHeight();
            for (int i = 0; i < numBalls; i++) {
                int totalColors = 6;
                int num = Greenfoot.getRandomNumber(totalColors);
                Ball.BallColor[] colors = Ball.BallColor.values();
                for (int j = 0; j < totalColors; j++) {
                    if (j == num) {
                        int x = (new Ball()).getImage().getWidth() /2 + i* b.getImage().getWidth();
                        addObject(new Ball(colors[j]), x, y);
                        this.numberOfBalls++;
                    }
                }
            }
        }
    }
    
    /**
     * Adds a number of points to the current score.
     */
    public void addScore(int score)
    {
        this.counter.setValue(this.counter.getValue() +score);
    }
    
    /**
     * This must be called if you 'eat' a ball.
     */
    public void ballEaten()
    {
        this.numberOfBalls--;
        if (this.numberOfBalls <= 0) {
            createBalls();
        }
    }
    
    /**
     * Let the game go game over.
     */
    public void gameOver(Actor sender)
    {
        if (sender instanceof Player) {
            removeObject(sender);
            highScoreThread = new Thread(new ScoreBoardThread(this, this.saver, this.counter.getValue()));
            highScoreThread.start();
            
            Button back = new Button(BACK_BUTTON_TITLE), replay = new Button(REPLAY_BUTTON_TITLE);
            back.setResponder(this);
            replay.setResponder(this);
            addObject(back, getWidth() /2 -75, getHeight() -50);
            addObject(replay, getWidth() /2 +75, getHeight() -50);
            
            removeObject(this.pauseButton);
            this.pauseButton = null;
        }
    }
    
    public void buttonClicked(Button button)
    {
        String buttonTitle = button.getTitle();
        if (buttonTitle.equals(BACK_BUTTON_TITLE)) {
            getResponder().worldShouldBeDismissed(this);
            highScoreThread.interrupt();
        } else if (buttonTitle.equals(REPLAY_BUTTON_TITLE)) {
            GameWorld newWorld = new GameWorld(getWidth(), getHeight(), getCellSize(), getBackground());
            newWorld.setResponder(getResponder());
            Greenfoot.setWorld(newWorld);
            highScoreThread.interrupt();
        } else if (buttonTitle.equals(PAUSE_BUTTON_TAG)) {
            if (this.pauseMenu == null) {
                this.pauseMenu = PauseMenu.showInWorld(this);
            } else {
                this.pauseMenu.removeFromWorld();
                gameShouldContinue(null);
            }
        }
    }
    
    public void gameShouldBePaused(PauseMenu sender)
    {
        this.paddle.setPaused(true);
        this.player.setPaused(true);
    }
    public void gameShouldContinue(PauseMenu sender)
    {
        this.paddle.setPaused(false);
        this.player.setPaused(false);
        this.pauseMenu = null;
    }
    public void gameShouldBeEnded(PauseMenu sender)
    {
        this.pauseMenu = null;
        getResponder().worldShouldBeDismissed(this);
    }
}