import greenfoot.*;


/**
 * A menu which is meant to pause the game. It doesn't know how to do it:
 * it has a responder for that.
 * 
 * @author Sven van Nigtevecht
 * @version 1.0
 */
public class PauseMenu extends Actor implements ButtonResponder
{
    private static final String RESUME_BUTTON_TITLE = "Resume";
    private static final String HOME_BUTTON_TITLE = "Main Menu";
    
    private PauseMenuResponder responder;
    private Actor[] objects;
    
    /**
     * Shows a pause menu in a given world. The given world will also
     * become the responder.
     * @return The pause menu that is displayed
     */
    public static PauseMenu showInWorld(World world)
    {
        PauseMenu menu = new PauseMenu();
        menu.setResponder((PauseMenuResponder) world);
        world.addObject(menu, world.getWidth() /2, world.getHeight() /2);
        return menu;
    }
    
    /**
     * Sets the responder of the pause menu.
     */
    public void setResponder(PauseMenuResponder responder)
    {
        if (this.responder != responder) {
            this.responder = responder;
        }
    }
    
    /**
     * Returns the responder of the pause menu.
     */
    public PauseMenuResponder getResponder()
    {
        return this.responder;
    }
    
    /**
     * Everything the pause menu should do when it's added to the world:
     * create the UI for it, and send a message to it's responder.
     */
    public void addedToWorld(World world)
    {
        createImage();
        addUI();
        if (this.responder != null) {
            this.responder.gameShouldBePaused(this);
        }
    }
    
    private void createImage()
    {
        GreenfootImage background = new GreenfootImage(getWorld().getWidth(), getWorld().getHeight());
        background.setColor(new Color(0, 0, 0, 100));
        background.fill();
        
        GreenfootImage img = new GreenfootImage(getWorld().getWidth() -200, getWorld().getHeight() -100);
        img.setColor(Color.BLACK);
        img.fill();
        img.setColor(new Color(0, 0, 255, 200));
        img.fillRect(4, 4, img.getWidth() - 8, img.getHeight() - 8);
        img.setTransparency(200);
        
        background.drawImage(img, 100, 50);
        setImage(background);
    }
    
    private void addUI()
    {
        Text text = new Text("Game paused");
        text.setHeight(50);
        text.setColor(Color.WHITE);
        getWorld().addObject(text, getX(), getY() -(getImage().getHeight() /2) +120);
        
        Button b = new Button(RESUME_BUTTON_TITLE);
        b.setResponder(this);
        getWorld().addObject(b, getX(), getY() -10);
        
        Button bu = new Button(HOME_BUTTON_TITLE);
        bu.setResponder(this);
        getWorld().addObject(bu, getX(), getY() +60);
        
        this.objects = new Actor[3];
        this.objects[0] = text;
        this.objects[1] = b;
        this.objects[2] = bu;
    }
    
    /**
     * Changes the location of the pause menu.
     * It also moves the buttons, and the text of the menu.
     */
    public void setLocation(int x, int y)
    {
        if (getX() != x && getY() != y) {
            int difX = x - getX();
            int difY = y - getY();
            super.setLocation(x, y);
            for (Actor a : this.objects) {
                a.setLocation(a.getX()  +difX, a.getY() +difY);
            }
        }
    }
    
    public void buttonClicked(Button button)
    {
        if (this.responder != null) {
            String buttonTitle = button.getTitle();
            if (buttonTitle.equals(RESUME_BUTTON_TITLE)) {
                removeFromWorld();
                this.responder.gameShouldContinue(this);
            } else if (buttonTitle.equals(HOME_BUTTON_TITLE)) {
                removeFromWorld();
                this.responder.gameShouldBeEnded(this);
            }
        }
    }
    
    /**
     * Removes the pause menu from the world it is in.
     */
    public void removeFromWorld()
    {
        if (getWorld() != null) {
            getWorld().removeObject(this.objects[0]);
            getWorld().removeObject(this.objects[1]);
            getWorld().removeObject(this.objects[2]);
            getWorld().removeObject(this);
        }
    }
}